@@ -8,7 +8,7 @@
 // Copyright 2014 Luigi Scarso <luigi.scarso@gmail.com>
 // Copyright 2014, 2017-2019, 2021, 2023 Albert Astals Cid <aacid@kde.org>
 // Copyright 2015 Dmytro Morgun <lztoad@gmail.com>
-// Copyright 2018, 2021 Adrian Johnson <ajohnson@redneon.com>
+// Copyright 2018, 2021, 2023 Adrian Johnson <ajohnson@redneon.com>
 // Copyright 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright 2018 Adam Reichold <adam.reichold@t-online.de>
 //
@@ -193,7 +193,7 @@ struct AttributeMapEntry
 
 struct AttributeDefaults
 {
-    AttributeDefaults() {}; // needed to support old clang
+    AttributeDefaults() { }; // needed to support old clang
 
     Object Inline = Object(objName, "Inline");
     Object LrTb = Object(objName, "LrTb");
@@ -210,20 +210,11 @@ struct AttributeDefaults
 
 static const AttributeDefaults attributeDefaults;
 
-#define ATTR_LIST_END                                                                                                                                                                                                                          \
-    {                                                                                                                                                                                                                                          \
-        Attribute::Unknown, nullptr, nullptr, false, nullptr                                                                                                                                                                                   \
-    }
+#define ATTR_LIST_END { Attribute::Unknown, nullptr, nullptr, false, nullptr }
 
-#define ATTR_WITH_DEFAULT(name, inherit, check, defval)                                                                                                                                                                                        \
-    {                                                                                                                                                                                                                                          \
-        Attribute::name, #name, &attributeDefaults.defval, inherit, check                                                                                                                                                                      \
-    }
+#define ATTR_WITH_DEFAULT(name, inherit, check, defval) { Attribute::name, #name, &attributeDefaults.defval, inherit, check }
 
-#define ATTR(name, inherit, check)                                                                                                                                                                                                             \
-    {                                                                                                                                                                                                                                          \
-        Attribute::name, #name, nullptr, inherit, check                                                                                                                                                                                        \
-    }
+#define ATTR(name, inherit, check) { Attribute::name, #name, nullptr, inherit, check }
 
 static const AttributeMapEntry attributeMapCommonShared[] = { ATTR_WITH_DEFAULT(Placement, false, isPlacementName, Inline),
                                                               ATTR_WITH_DEFAULT(WritingMode, true, isWritingModeName, LrTb),
@@ -779,6 +770,15 @@ bool StructElement::getPageRef(Ref &ref) const
     return false;
 }
 
+bool StructElement::getStmRef(Ref &ref) const
+{
+    if (stmRef.isRef()) {
+        ref = stmRef.getRef();
+        return true;
+    }
+    return false;
+}
+
 const char *StructElement::getTypeName() const
 {
     return typeToName(type);
