@@ -3,6 +3,7 @@
  * Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
  * Copyright (C) 2020, 2021 Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2021, Even Rouault <even.rouault@spatialys.com>
+ * Copyright (C) 2024, g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +44,11 @@ void QIODeviceOutStream::put(char c)
     m_device->putChar(c);
 }
 
+size_t QIODeviceOutStream::write(std::span<unsigned char> data)
+{
+    return m_device->write(reinterpret_cast<const char *>(data.data()), data.size());
+}
+
 static int poppler_vasprintf(char **buf_ptr, const char *format, va_list ap) GCC_PRINTF_FORMAT(2, 0);
 
 static int poppler_vasprintf(char **buf_ptr, const char *format, va_list ap)
